"""
    blc_core wrapper.
"""
from ctypes import *
from ctypes.util import find_library

path=find_library("blc_core")
lib = CDLL(name=path)

class BlcDim(Structure):
    _fields_ = [("length", c_size_t), ("step", c_size_t)]

class BlcArray:
    lib.blc_array_new.restype = c_void_p
    lib.blc_array_new_void.restype = c_void_p
    lib.blc_array_new_void.argtypes = []

    lib.blc_array_delete.argtypes = [ c_void_p ]
    types={ 'UIN8': c_uint8, 'INT8': c_int8, 'UI16': c_uint16, 'INT16': c_int16, 'UI32': c_uint32, 'IN32': c_int32, 'UI64': c_uint64, 'IN64': c_int64, 'FL32': c_float, 'FL64': c_double}

    def __init__(self,  data_type=None, format_type=None, dims=None):
        self.type = data_type
        self.format = format_type
        self.dims = dims
        
        if (dims): #Creates
            dim_types = [c_int] * len(dims)
            lib.blc_array_new.argtypes = [c_uint32, c_uint32, c_int, c_int, c_int ]
            
            # Create a new array
            self.pointer = lib.blc_array_new(int.from_bytes(data_type.encode(), "big"), int.from_bytes(format_type.encode(), "big"), 2, 64, 4)
        
            #Compute the length of the array
            self.total_length=1
            for length in dims:
                self.total_length *= length
        
            # Specify types for channel
            self.ctype = BlcArray.types[self.type]
            # Specify pointer of data for array
            self.data = cast(lib.blc_array_get_data(self.pointer), POINTER(self.ctype))
        else:
            self.pointer = lib.blc_array_new_void()
        
        
    def __del__(self):
        BlcArray.lib.blc_array_delete(self.array)

if __name__ == "__main__":
    
    undef_array = BlcArray()
    array = BlcArray('UIN8', 'TEXT', [16, 2])
    
    array.data[0]=32
    array.data[1]=8
    array.data[2]=0

    
    print("type",array.type, "format", array.format, "length", array.total_length)
    print(array.data[:array.total_length])

