
from ctypes import *
from ctypes.util import find_library

from blpy_core import BlcArray

# Bind blc_network methods
lib_path = find_library("blc_network")
lib = CDLL(lib_path)

class BlcArrayTCP4Server:
    lib.blc_array_tcp4_server_new.restype = c_void_p
    lib.blc_array_tcp4_server_new.argtypes = [ c_void_p, c_char_p ]
    lib.blc_array_tcp4_server_delete.argtypes = [ c_void_p ]

    def __init__(self, array, port):
        self.array = array
        self.port = port
        self.pointer = lib.blc_array_tcp4_server_new(array.pointer, port.encode())
    
    def wait_connection(self):
        error = lib.blc_array_tcp4_server_wait_connection(self.pointer)
        if (error != 0):
            raise Exception('Error in accept:'+ error) 
    
    def __del__(self):
        lib.blc_array_tcp4_server_delete(self.pointer)

class BlcArrayTCP4Client:
    lib.blc_array_tcp4_client_new.restype = c_void_p
    lib.blc_array_tcp4_client_new.argtypes = [ c_void_p, c_char_p, c_char_p ]
    lib.blc_array_tcp4_client_delete.argtypes = [ c_void_p ]

    def __init__(self, array, address, port):
        self.array = array
        self.port = port
        self.pointer = lib.blc_array_tcp4_client_new(array.pointer, address.encode(), port.encode())
    
    def __del__(self):
        blc_array_tcp4_client_delete(self.pointer)
   
                
if __name__ == "__main__":
    import threading
    
    class ServerThread (threading.Thread):
        def __init__(self, server):
            threading.Thread.__init__(self)
            self.server=server

        def run(self):
            print("Serveur has started, waiting for connection\n")
            self.server.wait_connection()
            print("Client connected")
            self.server.send_data()
    
    '''
    This thread is only for example, it is supposed to be executed in another process ( the server )
    '''
    serverArray = BlcArray('UIN8', 'TEXT', [5])
    serverArray.data[2]=3
    server = BlcArrayTCP4Server(serverArray, "31440")
    thread=ServerThread(server)
    print("Initilized thread")
    thread.start()
    print("Server started")
    
        
    '''Client part'''
    clientArray = BlcArray()
    client = BlcArrayTCP4Client(clientArray, "localhost", "31440")
    print("type",clientArray.type, "format", clientArray.format, "length", clientArray.total_length)

    print(clientArray.array[:clientArray.total_length])
    client.read_data()
    print(clientArray.array[:clientArray.total_length])
    
    thread.join()
    
    
    