#include "blc_array_tcp4_client.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <netinet/tcp.h>
#include <vector>

using namespace std;

blc_array_tcp4_client::blc_array_tcp4_client(blc_array *array, string const &address, string const &port_name):array(array){
    int ret;
    struct addrinfo hints, *results;

    CLEAR(hints);
    hints.ai_family = PF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;

    ret=getaddrinfo(address.c_str(), port_name.c_str(), &hints, &results);
    if (ret !=0) EXIT_ON_ERROR(gai_strerror(ret));
    if(results->ai_next != nullptr) EXIT_ON_ERROR("There is more than one possible address. It is not yet implemented.");

    SYSTEM_ERROR_CHECK(socket_fd = socket(results->ai_family, results->ai_socktype, 0), -1, nullptr);
    int flags =1;
    SYSTEM_ERROR_CHECK(setsockopt(socket_fd, IPPROTO_TCP, TCP_NODELAY, &flags, sizeof(flags)), -1, nullptr); //Disable Naggle algorithm. Packet are send even if they are small
    SYSTEM_ERROR_CHECK(connect(socket_fd, results->ai_addr, results->ai_addrlen), -1, nullptr);
    freeaddrinfo(results);
    if (array->dims==nullptr) read_properties();
    else EXIT_ON_ERROR("The local array is already defined.");
}

void blc_array_tcp4_client::read_properties(){
    ssize_t ret;
    vector<char> properties(1024);

    for (size_t read_nb=0; read_nb < properties.size(); read_nb+=static_cast<size_t>(ret)){
        SYSTEM_ERROR_CHECK(ret=read(socket_fd, properties.data()+read_nb, properties.size()-read_nb), -1, nullptr);
    }
    array->sscan_properties(properties.data()); // The array will need allocation
}

void blc_array_tcp4_client::read_data(){
    ssize_t ret;

    for (size_t read_nb=0; read_nb<array->size; read_nb+=static_cast<size_t>(ret)){
        SYSTEM_ERROR_CHECK(ret=read(socket_fd, array->chars+read_nb, array->size-read_nb), -1, nullptr);
    }
}

void blc_array_tcp4_client::send_data(){
    SYSTEM_ERROR_CHECK(write(socket_fd, array->data, array->size), -1, nullptr);
}

blc_array_tcp4_client::~blc_array_tcp4_client(){
    SYSTEM_ERROR_CHECK(close(socket_fd), -1, nullptr);
}


START_EXTERN_C
blc_array_tcp4_client *blc_array_tcp4_client_new(blc_array *array, const char *address, const char* port){
    return new blc_array_tcp4_client(array, address, port);
}

void blc_array_tcp4_client_delete(blc_array_tcp4_client *client){
    delete client;
}
END_EXTERN_C


