//
//  blc_tcp4.cpp
//  blc_network
//
//  Created by Arnaud Blanchard on 09/10/2020.
//


#include "blc_network.h"
#include "blc_mem.h"
#include "blc_realtime.h"

#include "blc_tcp4.h"


#include <unistd.h>
#include <pthread.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
 #include <net/if.h>
 #include <netinet/tcp.h> 

#include <thread>

using namespace std;

blc_tcp4_client::blc_tcp4_client( string const &address_name, string  const &port_name, ssize_t buffer_size){
    if (buffer_size==-1) allocate_system_size_buffer();
    else if (buffer_size!=0) allocate(buffer_size);
    
    create_ip4_socket(address_name.c_str(), port_name.c_str(), SOCK_STREAM);
    
    int flags =1;
    SYSTEM_ERROR_CHECK(setsockopt(socket_fd, IPPROTO_TCP, TCP_NODELAY, (void *)&flags, sizeof(flags)), -1, nullptr); //Disable Naggle algorithm. Packet are send even if they are small
    SYSTEM_ERROR_CHECK(connect(socket_fd, (const struct sockaddr *)&remote_address, remote_address_length), -1, nullptr);
}

void blc_tcp4_client::send_buffer(void const* buffer, size_t buffer_size){
    ssize_t ret;
    size_t system_buffer_size=0;
    socklen_t len=sizeof(system_buffer_size);
    
    ret=::send(socket_fd, buffer, buffer_size, NO_FLAG);
    if (ret==-1){
        SYSTEM_ERROR_CHECK(getsockopt(socket_fd, SOL_SOCKET, SO_SNDBUF, &system_buffer_size, &len), -1, nullptr);
        if (errno==EMSGSIZE) EXIT_ON_SYSTEM_ERROR("socket_fd '%d', request size '%u' SNDBUFFER '%d'", socket_fd, buffer_size, system_buffer_size );
    }
    else if ((size_t)ret!=buffer_size) EXIT_ON_ERROR("Only '%l' bytes sent instead of '%lu'", ret, size);
}

 void blc_tcp4_server::server_manager(){
    ssize_t ret;
    int client_socket_fd;
    
    if (data==NULL) EXIT_ON_ERROR("You have not allocated the receive buffer");
    SYSTEM_ERROR_CHECK(client_socket_fd = accept(socket_fd, (struct sockaddr *)&remote_address,  &remote_address_length), -1, nullptr);
    while(1)  //Il faudrait stopper en fonction d'une interruption sur le read
    {
        received_size=0;
        while(received_size<=minimal_read){ // Maybe need timeout
            SYSTEM_ERROR_CHECK(ret = read(client_socket_fd, chars +received_size , size-received_size), -1, "Error receiving data. fd '%d' buffer size '%lu'", socket_fd, size);
            received_size+=ret;
        }
        callback(this);
    }
}

blc_tcp4_server::blc_tcp4_server(string const &port_name,  void(*on_receive_data)(blc_tcp4_server*), void *arg, ssize_t buffer_size, uint64_t minimal_read):callback(on_receive_data), callback_arg(arg), minimal_read(minimal_read){
    if (buffer_size==-1) allocate_system_size_buffer();
    else if (buffer_size!=0) allocate(buffer_size);
    
    create_ip4_socket(nullptr, port_name.c_str(), SOCK_STREAM);
    SYSTEM_ERROR_CHECK(bind(socket_fd, (const struct sockaddr *)&remote_address, remote_address_length), -1, "Socket %d", socket_fd);
    SYSTEM_ERROR_CHECK(listen(socket_fd, 1), -1, "socket_fd", socket_fd); //1 connection at a time for now
    
    thread_ptr=make_unique<thread>(&blc_tcp4_server::server_manager, this);
}
